/*
 * gsm.c
 *
 * Created: 2016-05-03 22:47:17
 *  Author: Doom
 */ 


#define F_CPU 11059200UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include "serial.h"
#include "defs.h"
#include "gsm.h"
#include "comm.h"

#define GSM_SMS_LENGTH			128
#define GSM_NUM_LENGTH			12
#define GSM_END_OF_SMS			0x1A


#define GSM_SIM_INSTALLED			1
#define GSM_SIM_NOT_INSTALLED		0

#define GSM_POWER_ON				1
#define GSM_POWER_OFF				2
#define GSM_POWER_PROBLEM			3
#define GSM_POWER_CLEAR				3


struct {
	char step;
	char nextstep;
	int16_t delay;
	char connected;
	char sms[GSM_SMS_LENGTH + 1];
	char number[GSM_NUM_LENGTH + 1];
	char sindex;
	char stask;
	char led;
	char restart;
} gsm;

//-------------------------------------------
void GsmInit (void) {
	
	GSM_SIM_PORT |= (1<<GSM_SIM_DETECT);
	GSM_PORT |= (1<<GSM_PWRKEY | 1<<GSM_RING);
	GSM_DDR |= (1<<GSM_PWRKEY);
	
	gsm.delay = (GSM_PERIOD + GSM_PERIOD);
}

//-------------------------------------------
char GsmProcGet (void) {
	return gsm.step;
}

//-------------------------------------------
void GsmProc (void) {
	if (gsm.led == 0) LEDO_OFF;
	if (gsm.connected == 0) {
		LEDR_SET;
	} else {
		LEDR_OFF;
	}
	

	//----------------------------------------
	if (gsm.delay > 0) {
		gsm.delay--;
		
	} else {
		LEDO_SET;
		
		gsm.restart++;
		if (gsm.restart >= GSM_RESTART_CYCLES) {
			gsm.restart = 0;
			gsm.nextstep = 0;
		}
		
		gsm.delay = GSM_PERIOD;
		gsm.step = gsm.nextstep;
		SerialRBufferClear();
		
		if (gsm.step == 0) {
			gsm.connected = 0;
			gsm.led = 1;

			if (GsmPowerSwitch(GSM_POWER_ON) == GSM_POWER_ON) {
				GsmPowerSwitch(GSM_POWER_CLEAR);
				gsm.nextstep = 1;
				gsm.led = 0;
				gsm.delay = GSM_PERIOD + GSM_PERIOD;
			}
		}
		
		if (gsm.step == 1) {
			SerialWBufferQuick("ATE0\r");
		}
		if (gsm.step == 2) {
			SerialWBufferQuick("AT+CREG=1\r");
		}
		if (gsm.step == 3) {
			SerialWBufferQuick("AT+CMGF=1\r");
		}
		if (gsm.step == 4) {
			SerialWBufferQuick("AT+CMGD=1\r");
		}
		if (gsm.step == 5) {
			gsm.connected = 0;				
			LEDR_SET;
			gsm.led = 0;
			SerialWBufferQuick("AT+CREG?\r");
		}
		if (gsm.step == 6) {
			SerialWBufferQuick("AT+CMGR=1\r");
		}
		if (gsm.step == 7) {
			//GsmSmsPrepare(text, numer);
			GsmSmsSend();			
			if (GsmSmsSendCheck() == 0){	//failure
				gsm.nextstep = 5;
			}
			if (GsmSmsSendCheck() == 1){	//second step
				gsm.delay = (GSM_PERIOD / 2);
			}
			if (GsmSmsSendCheck() == 2){	//success
				gsm.delay = (GSM_PERIOD + GSM_PERIOD);
				gsm.nextstep = 5;
			}				
		}
	}

	//----------------------------------------
	if (SerialRBufferStatus() >= 2){
		LEDO_SET;

		if (gsm.step == 0) {}			//pwr on

		if (gsm.step == 1) {			//ATE0
			if (GsmCheckOk() == 1) {
				gsm.restart = 0;
				gsm.nextstep = 2;
				SerialRBufferClear();
			}
			if (GsmCheckOk() == 2) {
				gsm.nextstep = 0;				
			}
		}
	
		if (gsm.step == 2) {			//CREG=1
			if (GsmCheckOk() == 1) {
				gsm.nextstep = 3;
				SerialRBufferClear();
			}
			if (GsmCheckOk() == 2) {
				gsm.nextstep = 0;
			}
		}
		
		if (gsm.step == 3) {			//CMGF=1
			if (GsmCheckOk() == 1) {
				gsm.nextstep = 4;
				SerialRBufferClear();
			}
			if (GsmCheckOk() == 2) {
				gsm.nextstep = 0;
			}
		}
		
		if (gsm.step == 4) {			//CMGD=1
			if (GsmCheckOk() == 1) {
				gsm.nextstep = 5;
				SerialRBufferClear();
			}
		}
				
		if (gsm.step == 5) {			//CREG?
			if (GsmCheckOk() == 1) {
				if (GsmGetParamVal(2) == 0) {
					if (gsm.restart < (GSM_RESTART_CYCLES - 10))
						gsm.restart = (GSM_RESTART_CYCLES - 10);
					//gsm.restart = GSM_RESTART_CYCLES;
				} 
				if (GsmGetParamVal(2) == 1) {
					gsm.connected = 1;
					gsm.restart = 0;
					
					if (gsm.stask) {
						gsm.led = 1;
						gsm.nextstep = 7;					
					} else {
						gsm.nextstep = 6;					
					}	
				} 
				SerialRBufferClear();
			}
		}
		
		if (gsm.step == 6) {			//CMGR=1
			if (GsmCheckOk() == 1) {
				gsm.nextstep = 5;
				if (GsmCheckCmd("CMGR") == 1) {
					GsmGetSender();
					CommGetMsg(SerialRBufferCharIndex('\n',2) + 1);
					gsm.nextstep = 4;
				}
				
				SerialRBufferClear();
			}
		}
	}
}

//-------------------------------------------
char GsmCheckOk (void) {
	char i;
	i = SerialRBufferCharLastIndex('\r', 2);
	i += 2;
	if (SerialRBufferChar(i) == 'O') {
		i++;
		if (SerialRBufferChar(i) == 'K') return 1;
	}
	if (SerialRBufferChar(i) == 'E') {
		i++;
		if (SerialRBufferChar(i) == 'R') {
			i++;
			if (SerialRBufferChar(i) == 'R') return 2;
		}
	}
	return 0;
}

//-------------------------------------------
int16_t GsmGetParamVal (char index) {
	char i;
	if (index <= 1) {
		i = SerialRBufferCharIndex(':',1);
		i+=2;
	} else {
		i = SerialRBufferCharIndex(',',(index-1));
		i++;
	}	
	return SerialRBufferValue(i);
}

//-------------------------------------------
char GsmCheckCmd (char *command) {
	char i;	
	char pass = 1;
	
	i = SerialRBufferCharIndex('+',1);
	i++;		
	while (*command > 0) {
		if (SerialRBufferChar(i) != *command) {
			pass = 0;
			break;
		}
		i++;
		command++;
	}
	return pass;
}

//-------------------------------------------
char GsmGetSender (void) {
	char i;
	i = SerialRBufferCharIndex(',',1);
	i += 5;
	return GsmSmsNumSet(SerialRBufferPt(i));
}

//-------------------------------------------
char GsmSmsTextAdd (char new, char *src, char len) {
	char status = 0;
	if (new > 0) gsm.sindex = 0;	
	
	if (*src == 0) status = 1;
	while (status == 0) {
		if (gsm.sindex < GSM_SMS_LENGTH) {
			gsm.sms[gsm.sindex] = *src;
			gsm.sindex++;
			src++;
			gsm.sms[gsm.sindex] = 0;
			if (len > 0) {
				len--;
				if (len == 0) status = 1;
			}
		} else {
			status = 2;			//not enough sms space
		}
		if (len == 0) {
			if (*src == 0) status = 1;
		}		
	}
	return status;
}

//-------------------------------------------
char GsmSmsNumSet (char *src) {
	char status = 0;
	char index = 0;
	char temp;
	
	if (*src != '+') {
		gsm.number[0] = '+';
		gsm.number[1] = '4';
		gsm.number[2] = '8';
		index = 3;
	}
	
	while (status == 0) {
		temp = *src;
		if ((temp >= 0) && (temp <= 9)) {
			temp = (temp + '0');
		}
		if ((temp >= '0') && (temp <= '9')) {
			if (index < GSM_NUM_LENGTH) {
				gsm.number[index] = temp;
				index++;
				src++;
				gsm.number[index] = 0;
				if (index == GSM_NUM_LENGTH) status = 1;
			} else {
				status = 2;
			}
		} else {
			status = 3;
		}
	}
	return status;
}

//-------------------------------------------
char *GsmSmsNumGet (void) {
	return &gsm.number[3];
}

//-------------------------------------------
char GsmSmsSendCheck (void) {
	//if (gsm.stask >= 2) gsm.stask = 0;
	return gsm.stask;
}

//-------------------------------------------
char GsmSmsPrepare (char *src, char *num) {
	if (gsm.stask == 0) {
		if (num > 0) GsmSmsNumSet(num);
		GsmSmsTextAdd(1, src, 0);
		gsm.stask = 1;
		return 1;
	}
	return 0;
}

//-------------------------------------------
char GsmSmsSend (void) {
	char i;
	char correct = 0;
	static char prev;

	if (gsm.stask) {
		if (prev == 0) {
		
			for(i=3; i<GSM_NUM_LENGTH; i++) {
				if (gsm.number[i] == '0') correct++;
				//if ((gsm.number[i] < '0') || (gsm.number[i] > '9')) gsm.stask = 0; 
			}
		
			if (correct >= 9) gsm.stask = 0;
			if (gsm.connected == 0) gsm.stask = 0;
		
			if (gsm.stask == 1) {
				SerialWBufferNew();
				SerialWBufferAdd("AT+CMGS=\"");
				SerialWBufferAdd(&gsm.number[0]);
				SerialWBufferAdd("\"\r");
				SerialWBufferSend();
				prev = 1;
				gsm.stask = 1;
			}
		} else { 
			//if (prev > 0) {
			SerialWBufferNew();
			SerialWBufferAdd(&gsm.sms[0]);
			SerialWBufferAddChar(GSM_END_OF_SMS);
			SerialWBufferSend();
			prev = 0;
			gsm.stask = 0;
			return 2;
		}
	}
	return gsm.stask;
}

//-------------------------------------------
char GsmSimStatus(void) {
	if (GSM_SIM_DETECT_GET == 0) 
		return GSM_SIM_INSTALLED;
		
	return GSM_SIM_NOT_INSTALLED;
}

//-------------------------------------------
char GsmPowerSwitch(char action) {
	char status = 0;
	static char prev;
				
	//---------------------------------------
	if (GSM_PWRKEY_GET == 0) {			//if pressed pwrey
		GSM_PWRKEY_RELEASE;				//then leave pwrkey
	} else {

		//---------------------------------------
		if (GSM_STATUS_GET > 0) {
			status = GSM_POWER_ON;
		} else {
			status = GSM_POWER_OFF;
		}
		
		//---------------------------------------
		if (action == GSM_POWER_CLEAR) {
			prev = 0;
			action = 0;
		}
		
		//---------------------------------------
		if (prev > 0) {
			prev--;
		} else {
			if (action == GSM_POWER_ON) {				
				if (status == GSM_POWER_ON) {
					GSM_PWRKEY_PRESS;
					prev = 0;
					status = 0;
				}
				
				if (status == GSM_POWER_OFF) {
					if (GsmSimStatus() == GSM_SIM_INSTALLED) {
						GSM_PWRKEY_PRESS;
			 			prev = 3;
					} else {
						prev = 0;
						status = GSM_POWER_PROBLEM;
					}
				}
			}
	
			//---------------------------------------
			if (action == GSM_POWER_OFF) {				
				if (status == GSM_POWER_ON) {
					GSM_PWRKEY_PRESS;
				}
			}
		}
	}
	return status;
}

//-------------------------------------------
/*char GsmComp (char *src, char *model) {
	char status = 0;
	char pass = 0;
	char *tsrc;
	char *tmodel;
	
	while (status == 0)
	{
		if (*src > 0) {
			if (*src == *model) {
				pass = 1;
				tsrc = src;
				tmodel = model;
				
				while (*tmodel > 0) {
					if (*tsrc != *tmodel ) {
						pass = 0;
						break;
					}
					tsrc++;
					tmodel++;
				}
				if (pass == 1) {
					status = 1;
				}
			} else {
				src++;
			}
		} else {
			break;
		}
	}
	return status;
}*/
